@@ -189,6 +189,9 @@ def getparser():
         type=str, default={}, dest='verbosity_details',
         callback=verbose_about_callback,
         help=("Be noiser during a particular stage of generation."))
+    parser.add_option(
+        '--introspect-c-modules', default=False, action='store_true',
+        help=("Import and introspect any C modules found."))
     return parser
 
 def readConfigFile(options):
@@ -314,8 +317,8 @@ def main(args):
                     system.msg('addPackage', 'adding directory ' + path)
                     system.addPackage(path, prependedpackage)
                 else:
-                    system.msg('addModule', 'adding module ' + path)
-                    system.addModule(path, prependedpackage)
+                    system.msg('addModuleFromPath', 'adding module ' + path)
+                    system.addModuleFromPath(prependedpackage, path)
                 system.packages.append(path)
 
         # step 3: move the system to the desired state
