@@ -7,9 +7,11 @@ being documented -- a System is a bad of Documentables, in some sense.
 """
 
 import datetime
+import imp
 import os
 import posixpath
 import sys
+import types
 import __builtin__
 
 # originally when I started to write pydoctor I had this idea of a big
@@ -24,6 +26,14 @@ import __builtin__
 #   Classes can contain Functions (when they get called Methods) and Classes
 #   Functions can't contain anything.
 
+
+class DocLocation:
+    OWN_PAGE = 1
+    PARENT_PAGE = 2
+    # Nothing uses this yet.  Parameters will one day.
+    #UNDER_PARENT_DOCSTRING = 3
+
+
 class Documentable(object):
     """An object that can be documented.
 
@@ -35,11 +45,11 @@ class Documentable(object):
     @ivar parent: ...
     @ivar parentMod: ...
     @ivar name: ...
-    @ivar document_in_parent_page: ...
+    @ivar documentation_location: ...
     @ivar sourceHref: ...
     @ivar kind: ...
     """
-    document_in_parent_page = False
+    documentation_location = DocLocation.OWN_PAGE
     sourceHref = None
 
     @property
@@ -81,7 +91,7 @@ class Documentable(object):
         return "%s %r"%(self.__class__.__name__, self.fullName())
 
     def docsources(self):
-        """Objects that can be consisdered as a source of documentation.
+        """Objects that can be considered as a source of documentation.
 
         The motivating example for having multiple sources is looking at a
         superclass' implementation of a method for documentation for a
@@ -141,20 +151,14 @@ class Documentable(object):
         parts = dottedname.split('.')
         obj = self._resolveName(parts[0], verbose)
         if obj is None:
-            self.system.msg(
-                "resolveDottedName", "%s:%s invalid ref to %s" % (
-                self.fullName(), self.linenumber, dottedname),
-                thresh=-1)
             return obj
-        system = self.system
         for p in parts[1:]:
-            if p not in obj.contents:
-                self.system.msg(
-                    "resolveDottedName", "%s:%s invalid ref to %s" % (
-                        self.fullName(), self.linenumber, dottedname),
-                    thresh=-1)
+            if p not in obj._name2fullname:
                 return None
-            obj = obj.contents[p]
+            fn = obj._name2fullname[p]
+            if fn not in self.system.allobjects:
+                return None
+            obj = self.system.allobjects[fn]
         if verbose > 1:
             print dottedname, '->', obj.fullName(), 'in', self.fullName()
         return obj
@@ -272,8 +276,9 @@ class Class(Documentable):
 
 
 class Function(Documentable):
-    document_in_parent_page = True
+    documentation_location = DocLocation.PARENT_PAGE
     kind = "Function"
+    linenumber = 0
     def setup(self):
         super(Function, self).setup()
         if isinstance(self.parent, Class):
@@ -286,6 +291,14 @@ class Function(Documentable):
             if self.name in b.contents:
                 yield b.contents[self.name]
 
+
+class Attribute(Documentable):
+
+    linenumber = 0
+    kind = "Attribute"
+    documentation_location = DocLocation.PARENT_PAGE
+
+
 class PrivacyClass:
     """'enum' containing values indicating how private an object should be.
 
@@ -311,6 +324,7 @@ class System(object):
     Module = Module
     Package = Package
     Function = Function
+    Attribute = Attribute
     # not done here for circularity reasons:
     #defaultBuilder = astbuilder.ASTBuilder
     sourcebase = None
@@ -526,9 +540,7 @@ class System(object):
 
         mod.sourceHref = posixpath.join(mod.system.sourcebase, *trailing)
 
-    def addModule(self, modpath, parentPackage=None):
-        fname = os.path.basename(modpath)
-        modname = os.path.splitext(fname)[0]
+    def addModule(self, modpath, modname, parentPackage=None):
         mod = self.Module(self, modname, None, parentPackage)
         self.addObject(mod)
         self.progress(
@@ -539,6 +551,52 @@ class System(object):
         self.module_count += 1
         self.setSourceHref(mod)
 
+    def ensureModule(self, module_full_name):
+        if module_full_name in self.allobjects:
+            return self.allobjects[module_full_name]
+        if '.' in module_full_name:
+            parent_name, module_name = module_full_name.rsplit('.', 1)
+            parent_package = self.ensurePackage(parent_name)
+        else:
+            parent_package = None
+            module_name = module_full_name
+        module = self.Module(self, module_name, None, parent_package)
+        self.addObject(module)
+        return module
+
+    def ensurePackage(self, package_full_name):
+        if package_full_name in self.allobjects:
+            return self.allobjects[package_full_name]
+        if '.' in package_full_name:
+            parent_name, package_name = package_full_name.rsplit('.', 1)
+            parent_package = self.ensurePackage(parent_name)
+        else:
+            parent_package = None
+            package_name = package_full_name
+        package = self.Package(self, package_name, None, parent_package)
+        self.addObject(package)
+        return package
+
+    def _introspectThing(self, thing, parent, parentMod):
+        for k, v in thing.__dict__.iteritems():
+            if isinstance(v, (types.BuiltinFunctionType, type(dict.keys))):
+                f = self.Function(self, k, v.__doc__, parent)
+                f.parentMod = parentMod
+                f.decorators = None
+                f.argspec = ((), None, None, ())
+                self.addObject(f)
+            elif isinstance(v, type):
+                c = self.Class(self, k, v.__doc__, parent)
+                c.parentMod = parentMod
+                self.addObject(c)
+                self._introspectThing(v, c, parentMod)
+
+    def introspectModule(self, py_mod, module_full_name):
+        module = self.ensureModule(module_full_name)
+        module.docstring = py_mod.__doc__
+        self._introspectThing(py_mod, module, module)
+        print py_mod
+
     def addPackage(self, dirpath, parentPackage=None):
         if not os.path.exists(dirpath):
             raise Exception("package path %r does not exist!"
@@ -546,9 +604,13 @@ class System(object):
         if not os.path.exists(os.path.join(dirpath, '__init__.py')):
             raise Exception("you must pass a package directory to "
                             "addPackage")
-        package = self.Package(self, os.path.basename(dirpath),
-                               None, parentPackage)
-        self.addObject(package)
+        if parentPackage:
+            prefix = parentPackage.fullName() + '.'
+        else:
+            prefix = ''
+        package_name = os.path.basename(dirpath)
+        package_full_name = prefix + package_name
+        package = self.ensurePackage(package_full_name)
         package.filepath = dirpath
         self.setSourceHref(package)
         for fname in os.listdir(dirpath):
@@ -557,8 +619,29 @@ class System(object):
                 initname = os.path.join(fullname, '__init__.py')
                 if os.path.exists(initname):
                     self.addPackage(fullname, package)
-            elif fname.endswith('.py') and not fname.startswith('.'):
-                self.addModule(fullname, package)
+            elif not fname.startswith('.'):
+                self.addModuleFromPath(package, fullname)
+
+    def addModuleFromPath(self, package, path):
+        for (suffix, mode, type) in imp.get_suffixes():
+            if not path.endswith(suffix):
+                continue
+            module_name = os.path.basename(path[:-len(suffix)])
+            if type == imp.C_EXTENSION:
+                if not self.options.introspect_c_modules:
+                    continue
+                if package is not None:
+                    module_full_name = "%s.%s" % (
+                        package.fullName(), module_name)
+                else:
+                    module_full_name = module_name
+                py_mod = imp.load_module(
+                    module_full_name, open(path, 'rb'), path,
+                    (suffix, mode, type))
+                self.introspectModule(py_mod, module_full_name)
+            elif type == imp.PY_SOURCE:
+                self.addModule(path, module_name, package)
+            break
 
     def handleDuplicate(self, obj):
         '''This is called when we see two objects with the same
