@@ -264,3 +264,52 @@ def test_classmethod():
         f = classmethod(f)
     ''')
     assert mod.contents['C'].contents['f'].kind == 'Class Method'
+
+def test_classdecorator():
+    mod = fromText('''
+    @cd
+    class C:
+        pass
+    ''')
+    assert mod.contents['C'].decorators == [(('cd', 'cd', None), None)], \
+      mod.contents['C'].decorators
+    mod = fromText('''
+    @module.cd
+    class C:
+        pass
+    ''')
+    assert mod.contents['C'].decorators == [(('module.cd', 'module.cd', None), None)], \
+      mod.contents['C'].decorators
+
+    mod = fromText('''
+    def cd(cls):
+        pass
+    @cd
+    class C:
+        pass
+    ''', modname='mod')
+    C = mod.contents['C']
+    assert C.decorators == [(('cd', 'mod.cd', mod.contents['cd']), None)], \
+      C.decorators
+
+
+def test_classdecorator_with_args():
+    mod = fromText('''
+    @cd()
+    class C:
+        pass
+    ''', modname='test')
+    C = mod.contents['C']
+    assert C.decorators == [(('cd', 'cd', None), [])], \
+      C.decorators
+    mod = fromText('''
+    class A: pass
+    @cd(A)
+    class C:
+        pass
+    ''', modname='test')
+    A = mod.contents['A']
+    C = mod.contents['C']
+    assert C.decorators == [(('cd', 'cd', None), [('A', 'test.A', A)])], \
+      C.decorators
+
